@@ -30,7 +30,7 @@ class KDescendantsProxyModelPrivate
     Q_DECLARE_PUBLIC(KDescendantsProxyModel)
     KDescendantsProxyModel *const q_ptr;
 
-    mutable QVector<QPersistentModelIndex> m_pendingParents;
+    mutable QList<QPersistentModelIndex> m_pendingParents;
 
     void scheduleProcessPendingParents() const;
     void processPendingParents();
@@ -106,12 +106,12 @@ void KDescendantsProxyModelPrivate::scheduleProcessPendingParents() const
 void KDescendantsProxyModelPrivate::processPendingParents()
 {
     Q_Q(KDescendantsProxyModel);
-    const QVector<QPersistentModelIndex>::iterator begin = m_pendingParents.begin();
-    QVector<QPersistentModelIndex>::iterator it = begin;
+    const QList<QPersistentModelIndex>::iterator begin = m_pendingParents.begin();
+    QList<QPersistentModelIndex>::iterator it = begin;
 
-    const QVector<QPersistentModelIndex>::iterator end = m_pendingParents.end();
+    const QList<QPersistentModelIndex>::iterator end = m_pendingParents.end();
 
-    QVector<QPersistentModelIndex> newPendingParents;
+    QList<QPersistentModelIndex> newPendingParents;
 
     while (it != end && it != m_pendingParents.end()) {
         const QModelIndex sourceParent = *it;
@@ -344,13 +344,6 @@ void KDescendantsProxyModel::collapseSourceIndex(const QModelIndex &sourceIndex)
     Q_EMIT dataChanged(ownIndex, ownIndex, {ExpandedRole});
 }
 
-#if KITEMMODELS_BUILD_DEPRECATED_SINCE(4, 8)
-void KDescendantsProxyModel::setRootIndex(const QModelIndex &index)
-{
-    Q_UNUSED(index)
-}
-#endif
-
 QModelIndexList KDescendantsProxyModel::match(const QModelIndex &start, int role, const QVariant &value, int hits, Qt::MatchFlags flags) const
 {
     return QAbstractProxyModel::match(start, role, value, hits, flags);
@@ -359,7 +352,7 @@ QModelIndexList KDescendantsProxyModel::match(const QModelIndex &start, int role
 namespace
 {
 // we only work on DisplayRole for now
-static const QVector<int> changedRoles = {Qt::DisplayRole};
+static const QList<int> changedRoles = {Qt::DisplayRole};
 }
 
 void KDescendantsProxyModel::setDisplayAncestorData(bool display)
@@ -1033,7 +1026,7 @@ void KDescendantsProxyModelPrivate::sourceRowsRemoved(const QModelIndex &parent,
     }
     Mapping::right_iterator lowerBound = m_mapping.rightLowerBound(proxyStart);
     if (lowerBound == m_mapping.rightEnd()) {
-        int proxyRow = (lowerBound - 1).key();
+        int proxyRow = std::prev(lowerBound).key();
 
         for (int row = newEnd.row(); row >= 0; --row) {
             const QModelIndex newEndSibling = q->sourceModel()->index(row, column, parent);
@@ -1063,9 +1056,9 @@ void KDescendantsProxyModelPrivate::sourceRowsRemoved(const QModelIndex &parent,
         }
         return;
     }
-    const Mapping::right_iterator boundAbove = lowerBound - 1;
+    const Mapping::right_iterator boundAbove = std::prev(lowerBound);
 
-    QVector<QModelIndex> targetParents;
+    QList<QModelIndex> targetParents;
     targetParents.push_back(parent);
     {
         QModelIndex target = parent;
@@ -1122,7 +1115,7 @@ void KDescendantsProxyModelPrivate::sourceRowsRemoved(const QModelIndex &parent,
 
     if (parent.isValid()) {
         const QModelIndex oindex = q->mapFromSource(parent);
-        QVector<int> rolesChanged({KDescendantsProxyModel::ExpandableRole});
+        QList<int> rolesChanged({KDescendantsProxyModel::ExpandableRole});
 
         if (!q->sourceModel()->hasChildren(parent)) {
             rolesChanged << KDescendantsProxyModel::ExpandedRole;
